import java.util.Random;
/**
 * class Maze generates a random maze of specified size, then recursively seeks a solution.
 * 
 * @ RR
 * @ 2002/11/17
 */
public class Maze
{
    private int height, width;
    private int[][] grid;

    /**
     * Constructor for objects of class Maze
     */
    public Maze(int aHeight, int aWidth)
    { 
        height = aHeight;
        width = aWidth;
        grid = new int[height][width];
        this.fill();
    }

    /**
     * uses random numbers to fill the maze
     */
    
    private void fill()
    {    
       Random generator = new Random(); 
       for (int i = 0; i < height; i++) 
       { 
        for (int j = 0; j < width; j++) 
        {
            double x = generator.nextDouble(); 
            if (x < .35)
            {
                grid[i][j] = 0;               
            }
            else
            {
                grid[i][j] = 1;                              
            }
        }// end for j
       } // end for i
       grid[0][0] = 1; // but let's be sure we can start
// the maze is filled       
     }
          
    public void printMaze()
    {    
       System.out.println();       
       for (int i = 0; i < height; i++) 
       { 
            for (int j = 0; j < width; j++)             
            {
                System.out.print(grid[i][j]);             
            }
            System.out.println(); 
       }
       System.out.println();
     }
         
   public boolean solve(int row, int column)
   {
        boolean done = false;
        if ( valid(row, column) ) 
        {
            grid[row][column] = 3;  // has been visited
            if (row == grid.length - 1 && column == grid[0].length - 1)
            {
                done = true; // maze is solved
            }
            else
            {
                done = solve(row + 1, column); // look down...
                if (!done)
                {
                    done = solve(row, column + 1); //right...
                 }
                 if (!done)
                 {
                    done = solve(row - 1, column); //up...
                 }
                 if (!done)
                 {
                    done = solve(row, column - 1); //left
                 }            
            }
            if (done) // part of the final path
                grid[row][column] = 7;
        } // fi valid
        
        return done;
   } // method solve
   
   private boolean valid(int row, int column)
   {
        boolean result = false;
        // check if in bounds
        if ( row >=0 && row < grid.length && column >= 0 && column < grid[0].length )
        {
            // check if not blocked nor previously tried
            if( grid[row][column] ==1 )
                result = true;
        }
        
        return result;
   } // method valid
   
}
